/*
 * NAME
 *   ndx2ndb
 *     Tag atoms into MFFA potential group(s).
 * SYNOPSIS
 *   ndx2ndb -n index.ndx -o index.ndb
 * AUTHORS
 *   Jelger Risselada (h.j.risselada@rug.nl)
 *   Martti Louhivuori (m.j.louhivuori@rug.nl)
 * 
 * ---
 * Copyright (C) 2008  Jelger Risselada & Martti Louhivuori
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * (version 2) as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  
 * 02110-1301, USA.
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <math.h>
#include <ctype.h>

#include "macros.h"
#include "smalloc.h"
#include "typedefs.h"
#include "names.h"
#include "copyrite.h"
#include "statutil.h"
#include "tpxio.h"
#include "string2.h"
#include "strdb.h"
#include "vec.h"
#include "macros.h"
#include "index.h"
#include "pbc.h"
#include "xvgr.h"
#include "futil.h"
#include "matio.h"



/*
#include "statutil.h"
#include "typedefs.h"
#include "smalloc.h"
#include "vec.h"
#include "copyrite.h"
#include "statutil.h"
#include "tpxio.h"
#include <string.h>
#include <ctype.h>
#include "rdgroup.h"
*/


/* interface for different potentials */
void read_sdb_ndx(bool bNDB, char *filenm, int natoms, int ***SDB_IND)
{
	FILE *in;
	int   i, j, loop=0;
	
	snew(*SDB_IND, natoms);
	if (bNDB) {
		fprintf(stderr, "Reading index file...\n");
		in = ffopen(filenm, "r");
		for (i=0; i<natoms; i++) {
			fscanf(in, "%d\n", &loop);
			snew((*SDB_IND)[i], loop + 1);
			(*SDB_IND)[i][0] = loop;
			for (j=1; j<=loop; j++)
				fscanf(in,"%d\n", &(*SDB_IND)[i][j]);
		}
		ffclose(in);
	} else {
		for (i=0; i<natoms; i++) { 
			snew(*SDB_IND[i], 2);
			(*SDB_IND)[i][0] = 1;
			(*SDB_IND)[i][1] = 0;
		}
	}
	/*checking*/
	FILE *out;
	out = fopen("potential.log", "w");
	for (i=0; i<natoms; i++) {
		fprintf(out, "atom %i", i+1);
		for (j=1; j <= (*SDB_IND)[i][0]; j++)
			fprintf(out, " pot: %d",(*SDB_IND)[i][j]);
		fprintf(out, "\n");
	}
	fclose(out);
}

int gmx_ndx2ndb(int argc, char *argv[])
{
	static char *desc[] = {"Tag atoms into MFFA potential group(s)."};
	static int numb=2;
	t_atoms    atoms;
	char     **grpname;
	atom_id  **index;
	int       *size, max, i, j, k, **sdbdex, *check, next;
	int      **SDB_IND;
	FILE      *out;
	
	t_pargs pa[] = {
		{ "-numb", FALSE, etINT, {&numb}, 
			"Number of different potential groups."
		}
	};
	t_filenm fnm[] = {
		{ efNDX, "-n", NULL, ffREAD},
		{ efNDB, "-o", NULL, ffWRITE}  
	};
#define NFILE asize(fnm)
	
	CopyRight(stderr, argv[0]);
	
	parse_common_args(&argc, argv, PCA_CAN_VIEW | PCA_CAN_TIME | PCA_BE_NICE, 
			NFILE, fnm, asize(pa), pa, asize(desc), desc, 0, NULL);
	
	snew(grpname, numb);
	snew(index, numb);
	snew(size, numb);
	
	get_index(&atoms, ftp2fn(efNDX,NFILE,fnm), numb, size, index, grpname);
	out = ffopen(opt2fn("-o", NFILE, fnm), "w");
	/* find maximum */
	max=0;
	for (j=0; j<numb; j++)
		for (i=0; i<size[j]; i++)
			if (index[j][i] > max)
				max = index[j][i];
	max = max + 1;
	snew(sdbdex, max);
	for (i=0; i<max; i++)
		snew(sdbdex[i], numb + 1);
	
	/*distribute and sort index */
	for (j=0; j<numb; j++)
		for (i=0; i<size[j]; i++) {
			sdbdex[index[j][i]][0]++;
			next = sdbdex[index[j][i]][0];
			sdbdex[index[j][i]][next] = j;
		}
	for (i=0; i<max; i++) {
		for (j=0; j<=sdbdex[i][0]; j++)
			fprintf(out, "%d\n", sdbdex[i][j]);
	}
	fclose(out);
	
	/*This is the actual interface used in mdrun*/
	/*It opens the file again, reads it and print*/
	/*out the selected potentials for each atom*/
	read_sdb_ndx(TRUE, opt2fn("-o", NFILE, fnm), max, &SDB_IND);
	
	thanx(stderr);
	return 0;
}

